class Edge:
    def __init__(self, edge, transition_matrix=None):
        self.edge = edge
        self.transition_matrix = transition_matrix

class MM:
    def __init__(self, source, target, matrix):
        self.source = source
        self.target = target
        self.matrix = matrix


import numpy as np

from GenPhylo.model.math_model import generate_random_matrix
from GenPhylo.model.model_utils import DLC, compare_equal_matrices
from GenPhylo.utils.classes import Edge


def generate_transition_matrices(tree, node_distribution):
    """
    Generates transition matrices for each edge in the phylogenetic tree.
    """
    matrices = []
    for edge in tree.edges():
        l = 4 * edge[1].branch_length  # (Lake'94)
        matrix = np.zeros((4,4))
        while compare_equal_matrices(matrix, np.zeros((4,4))):
            matrix = generate_random_matrix(node_distribution[edge[0].name], l)
        assert (np.sum(matrix) > 0)
        iter = 0 # threshold on the number of iterations for DLC matrices 
        while not DLC(matrix) and iter < 5:
            iter += 1
            matrix = generate_random_matrix(node_distribution[edge[0].name], l)
        if iter == 5:
            print("Warning: Could not generate a DLC matrix for edge: ", edge)
        new_edge = Edge(edge, matrix)
        matrices.append(new_edge)
        node_distribution[edge[1].name] = np.matmul(node_distribution[edge[0].name], new_edge.transition_matrix)
        for i in range(4):
            assert (np.sum(new_edge.transition_matrix[i, :]) < 1.000000001 and np.sum(
                new_edge.transition_matrix[i, :]) > 0.999999999)
    return matrices


import numpy as np


def generate_root_distribution():
    """
    Generates a random root distribution until it meets the specified conditions.
    """
    B = False
    while not B:
        R = np.random.dirichlet([1, 1, 1, 1])
        Res = all(ele > 0.2 and ele < 0.3 for ele in R)
        if Res == True:
            B = True
    return R


def rename_nodes(tree):
    """
    Renames the nodes and leaves of the phylogenetic tree.
    """
    for idx, clade in enumerate(tree.get_nonterminals()):
        clade.name = "Node_" + str(idx) if idx > 0 else "Root"
    for idx, clade in enumerate(tree.get_terminals()):
        clade.name = "Leaf_" + clade.name


import json
import numpy as np
from Bio import Phylo
from io import StringIO

from GenPhylo.utils.misc_utils import generate_root_distribution, rename_nodes
from GenPhylo.utils.matrices_generation import generate_transition_matrices
from GenPhylo.utils.save import save_transition_matrices
from GenPhylo.utils.generate_DNA import generate_alignments

def process_tree(tree, root_distr):
    """
    Process the root distribution and the phylogenetic tree, both user inputs.
    """
    
    if root_distr == "random":
        root_distribution = {"Root": generate_root_distribution()}
    else:
        # Parse a given string separating numbers given by commas and puting them in a 1x4 vector
        root_distribution = np.array(root_distr)
        # Check this vector sums up to 1
        assert sum(root_distribution) == 1, "Root distribution does not sum up to 1"
        root_distribution = {"Root": root_distribution}
    path_t = tree
    tree_file = open(path_t, "r")
    tree = tree_file.read()
    tree = Phylo.read(StringIO(tree), "newick")
    rename_nodes(tree)
    # Write the tree in our format to latter save it at the top of the matrix file
    newick_with_labels = StringIO()
    Phylo.write(tree, newick_with_labels, "newick", format_branch_length='%0.2f')
    newick_with_labels_str = newick_with_labels.getvalue()
    net = Phylo.to_networkx(tree)
    return net, root_distribution, newick_with_labels_str


def get_N_alignments(tree, L, N, root_distr, name):
    """
    Main function to generate transition matrices and alignments.
    """
    
    net, root_distribution, newick_with_labels_str = process_tree(tree, root_distr)
    matrices = generate_transition_matrices(net, root_distribution)
    save_transition_matrices(matrices, name, newick_with_labels_str)
    return generate_alignments(net, root_distribution, matrices, L, N, 1, [], name)


def get_alignments_by_lengths(tree, lengths, root_distr, name):
    """
    Main function to generate transition matrices and alignments.
    """
    
    net, root_distribution, newick_with_labels_str = process_tree(tree, root_distr)
    matrices = generate_transition_matrices(net, root_distribution)
    save_transition_matrices(matrices, name, newick_with_labels_str)
    return generate_alignments(net, root_distribution, matrices, [], [], 2, lengths, name)


from setuptools import setup, find_packages

# Ensure the file is read using UTF-8 encoding to avoid encoding issues
with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='GenPhylo',  # Package name in lowercase
    version='1.1.1',  # Version
    author='Marta Casanellas, Martí Cortada Garcia, Adrià Diéguez Moscardó',
    author_email='marta.casanellas@upc.edu',
    description='GenPhylo: An open-source Python module for simulating alignments along phylogenetic trees',
    long_description=long_description,  # Ensure README.md exists
    long_description_content_type='text/markdown',  # Ensure this matches the README format
    url='https://github.com/GenPhyloProject/GenPhylo',  # Link to your GitHub repo
    packages=find_packages(),  # Automatically find and include your package
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',  # Specify Python version support
    install_requires=[
        # List your package dependencies here, e.g., 'numpy>=1.18.0'
    ],
    maintainers=[
        'Marta Casanellas <marta.casanellas@upc.edu>',
        'Martí Cortada Garcia <marti.cortada@estudiantat.upc.edu>',
        'Adrià Diéguez Moscardó <adria.dieguez@estudiantat.upc.edu>',
    ],
)

